/*
 * The following code is released under the terms of the IBM
 * Common Public License v.5.
 */
#include <jni.h>
#include <jawt_md.h>
#include <windows.h>
#include "SWT_AWT.h"

typedef jboolean (JNICALL *JAWT_GetAWT_t)(JNIEnv* env, JAWT* awt);

JAWT_GetAWT_t g_pfnJAWT_GetAWT = NULL;

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	if (g_pfnJAWT_GetAWT == NULL) {
		HINSTANCE hJawt = LoadLibrary("jawt");
		if (hJawt == NULL) {
			return FALSE;
		}

		g_pfnJAWT_GetAWT = (JAWT_GetAWT_t)GetProcAddress(hJawt, "_JAWT_GetAWT@8");

		if (g_pfnJAWT_GetAWT == NULL) {
			return FALSE;
		}
	}

	return TRUE;
}

void throwSwtException(JNIEnv* env, char* message)
{
	jclass jcSwtExceptionClass = (*env)->FindClass(env, "org/eclipse/swt/SWTException");

	/* Unable to find the new exception class, give up. */
	if (jcSwtExceptionClass == 0) {
		return;
	}

	(*env)->ThrowNew(env, jcSwtExceptionClass, message);
}

JNIEXPORT jint JNICALL Java_org_eclipse_swt_internal_awt_win32_SWT_1AWT_getHwndForCanvas
		(JNIEnv* env, jclass clazz, jobject canvas)
{
	JAWT awt;
    JAWT_DrawingSurface* ds;
    JAWT_DrawingSurfaceInfo* dsi;
	JAWT_Win32DrawingSurfaceInfo* w32Dsi;
    jint lock;
	jint hwnd;

	/* Get the JAWT interface */
    awt.version = JAWT_VERSION_1_3;
    if (g_pfnJAWT_GetAWT(env, &awt) == JNI_FALSE) {
        throwSwtException(env, "JAWT not found");
		return -1;
    }

	/* Get the drawing surface */
    ds = awt.GetDrawingSurface(env, canvas);
    if (ds == NULL) {
        throwSwtException(env, "NULL drawing surface");
        return -1;
    }

    /* Lock the drawing surface */
    lock = ds->Lock(ds);
    if((lock & JAWT_LOCK_ERROR) != 0) {
        awt.FreeDrawingSurface(ds);
        throwSwtException(env, "Error locking surface");
        return -1;
    }

    /* Get the drawing surface info */
    dsi = ds->GetDrawingSurfaceInfo(ds);
    if (dsi == NULL) {
        ds->Unlock(ds);
        awt.FreeDrawingSurface(ds);
        throwSwtException(env, "Error getting surface info");
        return -1;
    }

	w32Dsi = (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;

	hwnd = (jint)w32Dsi->hwnd;

    /* Free the drawing surface info */
    ds->FreeDrawingSurfaceInfo(dsi);

    /* Unlock the drawing surface */
    ds->Unlock(ds);

    /* Free the drawing surface */
    awt.FreeDrawingSurface(ds);

	return hwnd;
}